@@ -15,10 +15,12 @@
 #' @param breaks a numeric vector of break values to define isochrone areas,
 #' in minutes.
 #' @param exclude pass an optional "exclude" request option to the OSRM API.
-#' @param res number of points used to compute isochrones, one side of the square
-#' grid, the total number of points will be res*res. Increase res to obtain more
-#' detailed isochrones.
-#' @param returnclass deprecated.
+#' @param n number of points used to compute isochrones, possible values are
+#' c(100, 200, 500, 1000, 2000, 5000, 10000, 20000, 50000).
+#' @param res deprecated
+#' @param smooth if TRUE a moving window with a gaussian blur is applied to 
+#' durations. This option may be usefull to remove small patches of hard to 
+#' reach areas. The computed isochrones are less precise but better looking. 
 #' @param osrm.server the base URL of the routing server.
 #' getOption("osrm.server") by default.
 #' @param osrm.profile the routing profile to use, e.g. "car", "bike" or "foot"
@@ -64,27 +66,25 @@
 #' }
 #' }
 osrmIsochrone <- function(loc, breaks = seq(from = 0, to = 60, length.out = 7),
-                          exclude, res = 30, returnclass,
+                          exclude, n = 500, smooth = FALSE, res,
                           osrm.server = getOption("osrm.server"),
                           osrm.profile = getOption("osrm.profile")) {
   opt <- options(error = NULL)
   on.exit(options(opt), add = TRUE)
 
-  if (!missing(returnclass)) {
-    warning('"returnclass" is deprecated.', call. = FALSE)
-  }
-
   # input management
   loc <- input_route(x = loc, id = "loc", single = TRUE)
   oprj <- loc$oprj
   loc <- st_as_sf(data.frame(lon = loc$lon, lat = loc$lat),
-    coords = c("lon", "lat"), crs = 4326
+                  coords = c("lon", "lat"), crs = 4326
   )
   loc <- st_transform(loc, "epsg:3857")
-
+  
   # max distance management to see how far to extend the grid to get measures
   breaks <- unique(sort(breaks))
   tmax <- max(breaks)
+  
+  # gentle sleeptime & param for demo server
   if (osrm.profile %in% c("foot", "walk")) {
     speed <- 10 * 1000 / 60
   }
@@ -95,19 +95,22 @@ osrmIsochrone <- function(loc, breaks = seq(from = 0, to = 60, length.out = 7),
     speed <- 120 * 1000 / 60
   }
   dmax <- tmax * speed
-
-
+  
   # gentle sleeptime & param for demo server
   if (osrm.server != "https://routing.openstreetmap.de/") {
     sleeptime <- 0
-    deco <- 450
+    deco <- 999
   } else {
     sleeptime <- 1
     deco <- 75
   }
 
+  # get the resolution
+  res <- get_resolution(res = res, n = n)
   # create a grid to obtain measures
-  sgrid <- rgrid(loc = loc, dmax = dmax, res = res)
+  ogrid <- rgrid(loc = loc, dmax = dmax, res = res)
+  sgrid <- ogrid[sf::st_is_within_distance(ogrid, loc, dmax, sparse = FALSE), ]
+  
   # slice the grid to make several API calls
   lsgr <- nrow(sgrid)
   niter <- lsgr %/% deco
@@ -140,19 +143,21 @@ osrmIsochrone <- function(loc, breaks = seq(from = 0, to = 60, length.out = 7),
     listDur[[ltot]] <- dmat$durations
     listDest[[ltot]] <- dmat$destinations
   }
-
+  
   measure <- do.call(c, listDur)
   destinations <- do.call(rbind, listDest)
   # for testing purpose
   # return(list(destinations = destinations, measure = measure,
   #             sgrid = sgrid, res = res, tmax = tmax))
-
+  
+  
   # assign values to the grid
-  sgrid <- fill_grid(
+  g <- fill_grid(
     destinations = destinations, measure = measure,
-    sgrid = sgrid, res = res, tmax = tmax
+    sgrid = ogrid, res = res, tmax = tmax
   )
-  if (min(sgrid$measure) >= tmax + 1) {
+  
+  if (min(g$measure, na.rm = TRUE) > tmax) {
     warning(
       paste0(
         "An empty object is returned. ",
@@ -169,19 +174,40 @@ osrmIsochrone <- function(loc, breaks = seq(from = 0, to = 60, length.out = 7),
     )
     return(empty_res)
   }
+  
+  # All values not within breaks are set to tmax+1 
+  g[is.na(g$measure), "measure"] <- tmax + .1
+  g[is.nan(g$measure), "measure"] <- tmax + .1
+  g[is.infinite(g$measure), "measure"] <- tmax + .1
+  
+  if (isTRUE(smooth)) {
+    if (!requireNamespace("terra", quietly = TRUE)) {
+      stop(paste0(
+        "'terra' package is needed for this function to work.",
+        "Please install it."
+      ), call. = FALSE)
+    }
+    r <- terra::rast(g[, c("COORDX", "COORDY", "measure"), drop = TRUE], 
+                     crs = "epsg:3857")
+    k <- terra::res(r)[1] / 2
+    rr <- terra::disagg(x = r, fact = 4, method  =  "near")
+    mat <- terra::focalMat(x = rr, d = k, type = "Gauss")
+    g <- terra::focal(x = rr, w = mat, fun = mean, na.rm = TRUE)
+  }
+  
   # computes isopolygones
-  iso <- mapiso(x = sgrid, breaks = breaks, var = "measure")
-  # get rid of out of breaks polys
+  iso <- mapiso(x = g, breaks = breaks, var = "measure")
+  # get rid of out of max breaks polys
   iso <- iso[-nrow(iso), ]
   # fisrt line always start at 0
   iso[1, "isomin"] <- 0
-
+  
   # proj mgmnt
   if (!is.na(oprj)) {
     iso <- st_transform(x = iso, oprj)
   } else {
     iso <- st_transform(x = iso, 4326)
   }
-
+  
   return(iso)
 }
